# Generated by roxygen2: do not edit by hand

S3method(hellinger,Matrix)
S3method(hellinger,data.frame)
S3method(hellinger,matrix)
S3method(loadings,default)
S3method(loadings,varimaxes)
S3method(predict,varimaxes)
S3method(rotation,varimaxes)
S3method(scores,varimaxes)
S3method(tidy,tmfast)
S3method(tsne,STM)
S3method(tsne,data.frame)
S3method(tsne,tmfast)
S3method(umap,STM)
S3method(umap,matrix)
S3method(umap,tmfast)
export(build_matrix)
export(compare_betas)
export(draw_corpus)
export(entropy)
export(expected_entropy)
export(fit_varimax)
export(hellinger)
export(insert_topics)
export(journal_specific)
export(loadings)
export(ndH)
export(ndR)
export(peak_alpha)
export(rdirichlet)
export(renorm)
export(rotation)
export(scores)
export(solve_power)
export(target_power)
export(tidy)
export(tidy_all)
export(tmfast)
export(tsne)
export(umap)
export(varimax_irlba)
importFrom(Matrix,crossprod)
importFrom(Matrix,t)
importFrom(Matrix,tcrossprod)
importFrom(Matrix,which)
importFrom(generics,tidy)
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(stats,predict)
