% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/s3.R
\name{loadings}
\alias{loadings}
\alias{loadings.default}
\title{Extract a PCA/varimax loadings matrix}
\usage{
loadings(x, ...)

\method{loadings}{default}(x, ...)
}
\arguments{
\item{x}{Object to dispatch on}

\item{...}{Passed to methods}
}
\value{
An object of class \code{"loadings"} (from \pkg{stats}), structured as a
matrix with vocabulary terms as rows and varimax factors as columns. Values are
the loading (weight) of each term on each factor.
}
\description{
Extract a PCA/varimax loadings matrix
}
\examples{
\donttest{
set.seed(42)
theta  = rdirichlet(50, 1, k = 3)
phi    = rdirichlet(3, 0.1, k = 20)
corpus = draw_corpus(rep(50L, 50), theta, phi)
model  = tmfast(corpus, n = 3)
loadings(model, k = 3)
}
v = stats::varimax(matrix(runif(20), nrow = 5))
loadings(v)
}
