% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/s3.R
\name{scores}
\alias{scores}
\title{Extract item scores from a fitted PCA/varimax model}
\usage{
scores(x, ...)
}
\arguments{
\item{x}{Object to dispatch on}

\item{...}{Passed to methods}
}
\value{
A numeric matrix with documents as rows and varimax factors as columns.
Values are the factor score for each document on each factor.
}
\description{
Extract item scores from a fitted PCA/varimax model
}
\examples{
\donttest{
set.seed(42)
theta  = rdirichlet(50, 1, k = 3)
phi    = rdirichlet(3, 0.1, k = 20)
corpus = draw_corpus(rep(50L, 50), theta, phi)
model  = tmfast(corpus, n = 3)
scores(model, k = 3)
}
}
